@@ -32,9 +32,14 @@ from mercurial.bookmarks import listbookmarks, readcurrent, pushbookmark
 from mercurial.util import sha1
 from mercurial import hg
 
+DEBUG_GITIFYHG = os.environ.get("DEBUG_GITIFYHG", "").lower() == "on"
+
 
 def log(msg, level="DEBUG"):
-    sys.stderr.write('%s: %s\n' % (level, str(msg)))
+    '''The git remote operates on stdin and stdout, so all debugging information
+    has to go to stderr.'''
+    if DEBUG_GITIFYHG or level != "DEBUG":
+        sys.stderr.write(u'%s: %r\n' % (level, msg))
 
 
 def die(msg, *args):
@@ -43,7 +48,9 @@ def die(msg, *args):
 
 
 def output(msg=''):
-    #log("OUT: %s" % msg)
+    if isinstance(msg, unicode):
+        msg = msg.encode('utf-8')
+    log("OUT: %s" % msg)
     print(msg)
 
 
@@ -61,7 +68,7 @@ def gitmode(flags):
 
 
 def hgmode(mode):
-    modes = {'0100755': 'x', '0120000': 'l'}
+    modes = {'100755': 'x', '120000': 'l'}
     return modes.get(mode, '')
 
 
@@ -170,7 +177,7 @@ class GitRemoteParser(object):
             self.line = self.peek_stack.pop(0)
         else:
             self.line = sys.stdin.readline().strip()
-        #log("INPUT: %s" % self.line)
+        log("INPUT: %s" % self.line)
         return self.line
 
     def peek(self):
@@ -219,9 +226,9 @@ class GitRemoteParser(object):
 
 class HGRemote(object):
     def __init__(self, alias, url):
-        if hg.islocal(url):
+        if hg.islocal(url.encode('utf-8')):
             url = p(url).abspath()
-        gitdir = p(os.environ['GIT_DIR'])
+        gitdir = p(os.environ['GIT_DIR'].decode('utf-8'))
         self.remotedir = gitdir.joinpath('hg', alias)
         self.marks_git_path = self.remotedir.joinpath('marks-git')
         self.marks = HGMarks(self.remotedir.joinpath('marks-hg'))
@@ -241,14 +248,14 @@ class HGRemote(object):
         myui = ui()
         myui.setconfig('ui', 'interactive', 'off')
 
-        local_path = os.path.join(self.remotedir, 'clone')
-        if not os.path.exists(local_path):
-            self.peer, dstpeer = hg.clone(myui, {}, url,
-                local_path, update=False, pull=True)
+        local_path = self.remotedir.joinpath('clone')
+        if not local_path.exists():
+            self.peer, dstpeer = hg.clone(myui, {}, url.encode('utf-8'),
+                local_path.encode('utf-8'), update=False, pull=True)
             self.repo = dstpeer.local()
         else:
-            self.repo = hg.repository(myui, local_path)
-            self.peer = hg.peer(myui, {}, url)
+            self.repo = hg.repository(myui, local_path.encode('utf-8'))
+            self.peer = hg.peer(myui, {}, url.encode('utf-8'))
             self.repo.pull(self.peer, heads=None, force=True)
 
     def process(self):
@@ -268,15 +275,15 @@ class HGRemote(object):
     def do_capabilities(self, parser):
         '''Process the capabilities request when incoming from git-remote.
         '''
-        output("import")
-        output("export")
-        output("refspec refs/heads/branches/*:%s/branches/*" % self.prefix)
-        output("refspec refs/heads/*:%s/bookmarks/*" % self.prefix)
-        output("refspec refs/tags/*:%s/tags/*" % self.prefix)
+        output(u"import")
+        output(u"export")
+        output(u"refspec refs/heads/branches/*:%s/branches/*" % self.prefix)
+        output(u"refspec refs/heads/*:%s/bookmarks/*" % self.prefix)
+        output(u"refspec refs/tags/*:%s/tags/*" % self.prefix)
 
         if self.marks_git_path.exists():
-            output("*import-marks %s" % self.marks_git_path)
-        output("*export-marks %s" % self.marks_git_path)
+            output(u"*import-marks %s" % self.marks_git_path)
+        output(u"*export-marks %s" % self.marks_git_path)
 
         output()
 
@@ -656,7 +663,8 @@ def main():
     '''Main entry point for the git-remote-gitifyhg command. Parses sys.argv
     and constructs a parser from the result.
     '''
-    HGRemote(*sys.argv[1:3]).process()
+    log(repr(sys.argv))
+    HGRemote(*[x.decode('utf-8') for x in sys.argv[1:3]]).process()
     try:
         sys.stderr.close()
     except:
